<?php

namespace Drupal\cl_block\Plugin\EmbedType;

use Drupal\cl_components\Component\Component;
use Drupal\cl_components\Component\ComponentDiscovery;
use Drupal\Core\Form\FormStateInterface;
use Drupal\embed\EmbedType\EmbedTypeBase;

/**
 * Principal Design System Element embed type.
 *
 * @EmbedType(
 *   id = "cl_block",
 *   label = @Translation("CL Component"),
 * )
 */
class ClComponentEmbed extends EmbedTypeBase {

  /**
   * {@inheritdoc}
   */
  public function getDefaultIconUrl() {
    $resolver = \Drupal::service('extension.path.resolver');
    $url_generator = \Drupal::service('file_url_generator');
    $path = $resolver->getPath('module', 'cl_block') . '/js/plugins/clcomponents/component.svg';
    return $url_generator->generateAbsoluteString($path);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return ['component_id' => ''];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['component_id'] = [
      '#type' => 'select',
      '#title' => $this->t('CL Component'),
      '#options' => $this->getComponentOptions(),
      '#default_value' => $this->configuration['component_id'],
      '#empty_option' => $this->t('- Select one -'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['component_id'] = $form_state->getValue('component_id');
  }

  /**
   * Gets the available component options.
   *
   * @return array
   *   The options array ready for a select.
   */
  private function getComponentOptions(): array {
    $components = \Drupal::service(ComponentDiscovery::class)->findAll();
    $options = array_reduce(
      $components,
      static function (array $carry, Component $component) {
        $group = $component->getMetadata()->getGroup();
        $name = $component->getMetadata()->getName();
        $id = $component->getId();
        if (empty($carry[$group])) {
          $carry[$group] = [];
        }
        $carry[$group][$id] = $name;
        return $carry;
      },
      []
    );
    ksort($options);
    foreach ($options as $group => $opts) {
      natsort($options[$group]);
    }
    return $options;
  }

}
